import time

from .event import Event
from .key import PrivateKey


def zero_bits(b: int) -> int:
    n = 0

    if b == 0:
        return 8

    while b >> 1:
        b = b >> 1
        n += 1

    return 7 - n


def count_leading_zero_bits(hex_str: str) -> int:
    total = 0
    for i in range(0, len(hex_str) - 2, 2):
        bits = zero_bits(int(hex_str[i : i + 2], 16))
        total += bits

        if bits != 8:
            break

    return total


def mine_event(
    content: str, difficulty: int, public_key: str, kind: int, tags: list = None
) -> Event:
    all_tags = [["nonce", "1", str(difficulty)]]
    if tags:
        all_tags.extend(tags)

    created_at = int(time.time())
    event_id = Event.compute_id(public_key, created_at, kind, all_tags, content)
    num_leading_zero_bits = count_leading_zero_bits(event_id)

    attempts = 1
    while num_leading_zero_bits < difficulty:
        attempts += 1
        all_tags[0][1] = str(attempts)
        created_at = int(time.time())
        event_id = Event.compute_id(public_key, created_at, kind, all_tags, content)
        num_leading_zero_bits = count_leading_zero_bits(event_id)

    return Event(public_key, content, created_at, kind, all_tags, event_id)


def mine_key(difficulty: int) -> PrivateKey:
    sk = PrivateKey()
    num_leading_zero_bits = count_leading_zero_bits(sk.public_key.hex())

    while num_leading_zero_bits < difficulty:
        sk = PrivateKey()
        num_leading_zero_bits = count_leading_zero_bits(sk.public_key.hex())

    return sk



from nostr.event import EncryptedDirectMessage, Event
from nostr.key import PrivateKey


def test_eq_true():
    """__eq__ should return True when PrivateKeys are equal."""
    pk1 = PrivateKey()
    pk2 = PrivateKey(pk1.raw_secret)
    assert pk1 == pk2


def test_eq_false():
    """__eq__ should return False when PrivateKeys are not equal."""
    pk1 = PrivateKey()
    pk2 = PrivateKey()
    assert pk1.raw_secret != pk2.raw_secret
    assert pk1 != pk2


def test_from_nsec():
    """PrivateKey.from_nsec should yield the source's raw_secret."""
    pk1 = PrivateKey()
    pk2 = PrivateKey.from_nsec(pk1.bech32())
    assert pk1.raw_secret == pk2.raw_secret


class TestEvent:
    def setup_class(self):
        self.sender_pk = PrivateKey()
        self.sender_pubkey = self.sender_pk.public_key.hex()

    def test_sign_event_is_valid(self):
        """Sign should create a signature that can be verified against Event.id."""
        event = Event(content="Hello, world!")
        self.sender_pk.sign_event(event)
        assert event.verify()

    def test_sign_event_adds_pubkey(self):
        """Sign should add the sender's pubkey if not already specified."""
        event = Event(content="Hello, world!")

        # The event's public_key hasn't been specified yet
        assert event.public_key is None

        self.sender_pk.sign_event(event)

        # PrivateKey.sign() should have populated public_key
        assert event.public_key == self.sender_pubkey


class TestEncryptedDirectMessage:
    def setup_class(self):
        self.sender_pk = PrivateKey()
        self.sender_pubkey = self.sender_pk.public_key.hex()
        self.recipient_pk = PrivateKey()
        self.recipient_pubkey = self.recipient_pk.public_key.hex()

    def test_encrypt_dm(self):
        """Should encrypt a DM and populate its `content` field with ciphertext that
        either party can decrypt."""
        message = "My secret message!"

        dm = EncryptedDirectMessage(
            recipient_pubkey=self.recipient_pubkey,
            cleartext_content=message,
        )

        # DM's content field should be initially blank
        assert dm.content is None
        self.sender_pk.encrypt_dm(dm)

        # After encrypting, the content field should now be populated
        assert dm.content is not None

        # Sender should be able to decrypt
        decrypted_message = self.sender_pk.decrypt_message(
            encoded_message=dm.content, public_key_hex=self.recipient_pubkey
        )
        assert decrypted_message == message

        # Recipient should be able to decrypt by referencing the sender's pubkey
        decrypted_message = self.recipient_pk.decrypt_message(
            encoded_message=dm.content, public_key_hex=self.sender_pubkey
        )
        assert decrypted_message == message

    def test_sign_encrypts_dm(self):
        """`sign` should encrypt a DM that hasn't been encrypted yet."""
        dm = EncryptedDirectMessage(
            recipient_pubkey=self.recipient_pubkey,
            cleartext_content="Some DM message",
        )

        assert dm.content is None

        self.sender_pk.sign_event(dm)

        assert dm.content is not None



import json
import time
import uuid

import click

from nostr.event import EncryptedDirectMessage, Event, EventKind
from nostr.filter import Filter, Filters
from nostr.key import PrivateKey, PublicKey
from nostr.message_type import ClientMessageType
from nostr.relay_manager import RelayManager


@click.group()
def cli():
    """Command related to message(s)."""


@cli.command()
@click.option("-p", "--pub-key", "npub", required=True, type=str)
@click.option("-l", "--limit", "limit", type=int, default=10)
@click.option("--sleep", "sleep", type=int, default=2)
def receive(npub: str, limit: int = 10, sleep: int = 2):
    """Receives messages from npub address."""
    click.echo(f"npub: {npub}")
    author = PublicKey.from_npub(npub)
    filters = Filters(
        [Filter(authors=[author.hex()], kinds=[EventKind.TEXT_NOTE], limit=limit)]
    )
    subscription_id = uuid.uuid1().hex
    request = [ClientMessageType.REQUEST, subscription_id]
    request.extend(filters.to_json_array())

    relay_manager = RelayManager()
    relay_manager.add_relay("wss://nostr-pub.wellorder.net")
    relay_manager.add_relay("wss://relay.damus.io")
    relay_manager.add_subscription(subscription_id, filters)

    with relay_manager:
        time.sleep(sleep)  # allow the connections to open
        message = json.dumps(request)
        relay_manager.publish_message(message)
        time.sleep(sleep)  # allow the messages to send
        while relay_manager.message_pool.has_events():
            event_msg = relay_manager.message_pool.get_event()
            print(event_msg.event.content)
        while relay_manager.message_pool.has_notices():
            notice_msg = relay_manager.message_pool.get_notice()
            print(notice_msg.content)


@cli.command()
@click.option("-s", "--sec-key", "nsec", required=True, type=str)
@click.option("-m", "--message", "message", type=str)
@click.option("--sleep", "sleep", type=int, default=2)
def publish(nsec: str, message: str, sleep: int = 2):
    """Sends a message."""
    private_key = PrivateKey.from_nsec(nsec)
    event = Event(content=message, public_key=private_key.public_key.hex())
    private_key.sign_event(event)

    message = json.dumps([ClientMessageType.EVENT, event.to_dict()])

    relay_manager = RelayManager()
    relay_manager.add_relay("wss://nostr-pub.wellorder.net")
    relay_manager.add_relay("wss://relay.damus.io")
    with relay_manager:
        time.sleep(sleep)  # allow the connections to open
        relay_manager.publish_message(message)


@cli.command()
@click.option("-s", "--sec-key", "nsec", required=True, type=str)
@click.option("-m", "--message", "message", type=str)
@click.option("-p", "--pub-key", "receiver_npub", required=True, type=str)
@click.option("--sleep", "sleep", type=int, default=2)
def send(nsec: str, message: str, receiver_npub: str, sleep: int = 2):
    """Sends a encryped direct message."""
    private_key = PrivateKey.from_nsec(nsec)
    recipient_pubkey = PublicKey.from_npub(receiver_npub)
    direct_message = EncryptedDirectMessage(
        recipient_pubkey=recipient_pubkey.hex(), cleartext_content=message
    )
    private_key.sign_event(direct_message)

    relay_manager = RelayManager()
    relay_manager.add_relay("wss://nostr-pub.wellorder.net")
    relay_manager.add_relay("wss://relay.damus.io")

    with relay_manager:
        time.sleep(sleep)  # allow the connections to open
        relay_manager.publish_event(direct_message)



import time
import unittest
from unittest.mock import ANY

import pytest

from nostr.event import EncryptedDirectMessage, Event
from nostr.key import PrivateKey


class TestEvent(unittest.TestCase):
    def test_event_default_time(self):
        """
        Ensure created_at default value reflects the time at Event object instantiation
        see: https://github.com/jeffthibault/python-nostr/issues/23
        """
        event1 = Event(content="test event")
        time.sleep(1.5)
        event2 = Event(content="test event")
        assert event1.created_at < event2.created_at

    def test_content_only_instantiation(self):
        """Should be able to create an Event by only specifying content without
        kwarg."""
        event = Event("Hello, world!")
        assert event.content is not None

    def test_event_id_recomputes(self):
        """Should recompute the Event.id to reflect the current Event attrs."""
        event = Event(content="some event")

        # id should be computed on the fly
        event_id = event.id

        event.created_at += 10

        # Recomputed id should now be different
        assert event.id != event_id

    def test_add_event_ref(self):
        """Should add an 'e' tag for each event_ref added."""
        some_event_id = "some_event_id"
        event = Event(content="Adding an 'e' tag")
        event.add_event_ref(some_event_id)
        assert ["e", some_event_id] in event.tags

    def test_add_pubkey_ref(self):
        """Should add a 'p' tag for each pubkey_ref added."""
        some_pubkey = "some_pubkey"
        event = Event(content="Adding a 'p' tag")
        event.add_pubkey_ref(some_pubkey)
        assert ["p", some_pubkey] in event.tags

    def test_dict_roundtrip(self):
        """Conversion to dict and back result in same object."""
        expected = {
            "content": "test event",
            "pubkey": "pubkey_any",
            "created_at": 12345678,
            "kind": 1,
            "tags": [],
            "sig": "signature",
        }
        event = Event.from_dict(expected)
        actual = event.to_dict()
        self.assertEqual(actual, {**expected, **{"id": ANY}})


class TestEncryptedDirectMessage:
    def setup_class(self):
        self.sender_pk = PrivateKey()
        self.sender_pubkey = self.sender_pk.public_key.hex()
        self.recipient_pk = PrivateKey()
        self.recipient_pubkey = self.recipient_pk.public_key.hex()

    def test_content_field_moved_to_cleartext_content(self):
        """Should transfer `content` field data to `cleartext_content`"""
        dm = EncryptedDirectMessage(
            content="My message!", recipient_pubkey=self.recipient_pubkey
        )
        assert dm.content is None
        assert dm.cleartext_content is not None

    def test_nokwarg_content_allowed(self):
        """Should allow creating a new DM w/no `content` nor `cleartext_content`
        kwarg."""
        dm = EncryptedDirectMessage(
            "My message!", recipient_pubkey=self.recipient_pubkey
        )
        assert dm.cleartext_content is not None

    def test_recipient_p_tag(self):
        """Should generate recipient 'p' tag."""
        dm = EncryptedDirectMessage(
            cleartext_content="Secret message!", recipient_pubkey=self.recipient_pubkey
        )
        assert ["p", self.recipient_pubkey] in dm.tags

    def test_unencrypted_dm_has_undefined_id(self):
        """Should raise Exception if `id` is requested before DM is encrypted."""
        dm = EncryptedDirectMessage(
            cleartext_content="My message!", recipient_pubkey=self.recipient_pubkey
        )

        with pytest.raises(Exception) as e:
            dm.id
        assert "undefined" in str(e)

        # But once we encrypt it, we can request its id
        self.sender_pk.encrypt_dm(dm)
        assert dm.id is not None



import json
import ssl
import threading
from dataclasses import dataclass
from threading import Lock

from .event import Event
from .filter import Filters
from .message_pool import MessagePool
from .relay import Relay, RelayPolicy, RelayProxyConnectionConfig
from .request import Request


class RelayException(Exception):
    pass


@dataclass
class RelayManager:
    def __post_init__(self):
        self.relays: dict[str, Relay] = {}
        self.message_pool: MessagePool = MessagePool()
        self.lock: Lock = Lock()

    def add_relay(
        self,
        url: str,
        policy: RelayPolicy = RelayPolicy(),
        ssl_options: dict = None,
        proxy_config: RelayProxyConnectionConfig = None,
    ):
        relay = Relay(url, policy, self.message_pool)

        with self.lock:
            self.relays[url] = relay

    def remove_relay(self, url: str):
        with self.lock:
            if url in self.relays:
                relay = self.relays.pop(url)
                relay.close()

    def add_subscription(self, id: str, filters: Filters):
        with self.lock:
            for relay in self.relays.values():
                if relay.policy.should_read:
                    relay.add_subscription(id, filters)

    def close_subscription(self):
        for relay in self.relays.values():
            relay.close_subscription(id)

    def __enter__(self):
        # NOTE: This disables ssl certificate verification
        self.open_connections({"cert_reqs": ssl.CERT_NONE})

    def open_connections(self, ssl_options: dict = None):
        for relay in self.relays.values():
            threading.Thread(target=relay.connect, name=f"{relay.url}-thread").start()

    def __exit__(self, type, value, traceback):
        self.close_connections()

    def close_connections(self):
        for relay in self.relays.values():
            relay.close()

    def publish_message(self, message: str):
        for relay in self.relays.values():
            if relay.policy.should_write:
                relay.publish(message)

    def get_connection_status(self):
        out = []
        for relay in self.relays.values():
            out.append([relay.url, relay.active])
        return out

    def add_subscription_on_relay(self, url: str, id: str, filters: Filters):
        with self.lock:
            if url in self.relays:
                relay = self.relays[url]
                if not relay.policy.should_read:
                    raise RelayException(
                        f"Could not send request: {relay.url} \
                        is not configured to read from"
                    )
                relay.add_subscription(id, filters)
                request = Request(id, filters)
                relay.publish(request.to_message())
            else:
                raise RelayException(f"Invalid relay url: no connection to {relay.url}")

    def add_subscription_on_all_relays(self, id: str, filters: Filters):
        with self.lock:
            for relay in self.relays.values():
                if relay.policy.should_read:
                    relay.add_subscription(id, filters)
                    request = Request(id, filters)
                    relay.publish(request.to_message())

    def close_subscription_on_relay(self, url: str, id: str):
        with self.lock:
            if url in self.relays:
                relay = self.relays[url]
                relay.close_subscription(id)
                relay.publish(json.dumps(["CLOSE", id]))
            else:
                raise RelayException(f"Invalid relay url: no connection to {relay.url}")

    def close_subscription_on_all_relays(self, id: str):
        with self.lock:
            for relay in self.relays.values():
                relay.close_subscription(id)
                relay.publish(json.dumps(["CLOSE", id]))

    def close_all_relay_connections(self):
        with self.lock:
            for url in self.relays:
                relay = self.relays[url]
                relay.close()

    def publish_event(self, event: Event):
        """Verifies that the Event is publishable before submitting it to relays."""
        if event.signature is None:
            raise RelayException(f"Could not publish {event.id}: must be signed")

        if not event.verify():
            raise RelayException(
                f"Could not publish {event.id}: \
                failed to verify signature {event.signature}"
            )

        with self.lock:
            for relay in self.relays.values():
                if relay.policy.should_write:
                    relay.publish(event.to_message())



import pytest

from nostr.event import Event
from nostr.key import PrivateKey
from nostr.relay_manager import RelayException, RelayManager


def test_only_relay_valid_events():
    """Publish_event raise a RelayException if an Event fails verification."""
    pk = PrivateKey()
    event = Event(
        public_key=pk.public_key.hex(),
        content="Hello, world!",
    )

    relay_manager = RelayManager()

    # Deliberately forget to sign the Event
    with pytest.raises(RelayException) as e:
        relay_manager.publish_event(event)
    assert "must be signed" in str(e)

    # Attempt to relay with a nonsense signature
    event.signature = "0" * 32
    with pytest.raises(RelayException) as e:
        relay_manager.publish_event(event)
    assert "failed to verify" in str(e)

    # Properly signed Event can be relayed
    pk.sign_event(event)
    relay_manager.publish_event(event)



import json
import time
from dataclasses import dataclass, field
from enum import IntEnum
from hashlib import sha256
from typing import List, Optional

from secp256k1 import PrivateKey, PublicKey

from .message_type import ClientMessageType


class EventKind(IntEnum):
    SET_METADATA = 0
    TEXT_NOTE = 1
    RECOMMEND_RELAY = 2
    CONTACTS = 3
    ENCRYPTED_DIRECT_MESSAGE = 4
    DELETE = 5
    BOOST = 6
    REACTION = 7


@dataclass
class Event:
    content: Optional[str] = None
    public_key: Optional[str] = None
    created_at: Optional[int] = None
    kind: Optional[int] = EventKind.TEXT_NOTE
    tags: List[List[str]] = field(default_factory=list)
    signature: Optional[str] = None

    def __post_init__(self) -> None:
        if self.content and not isinstance(self.content, str):
            raise TypeError("Argument 'content' must be of type str")

        if self.created_at is None:
            self.created_at = int(time.time())

    @staticmethod
    def serialize(
        public_key: str, created_at: int, kind: int, tags: List[List[str]], content: str
    ) -> bytes:
        data = [0, public_key, created_at, kind, tags, content]
        data_str = json.dumps(data, separators=(",", ":"), ensure_ascii=False)
        return data_str.encode()

    @staticmethod
    def compute_id(
        public_key: str, created_at: int, kind: int, tags: List[List[str]], content: str
    ):
        return sha256(
            Event.serialize(public_key, created_at, kind, tags, content)
        ).hexdigest()

    @property
    def id(self) -> str:
        # Always recompute the id to reflect the up-to-date state of the Event
        return Event.compute_id(
            self.public_key, self.created_at, self.kind, self.tags, self.content
        )

    def sign(self, private_key_hex: str) -> None:
        sk = PrivateKey(bytes.fromhex(private_key_hex))
        sig = sk.schnorr_sign(bytes.fromhex(self.id), None, raw=True)
        self.signature = sig.hex()

    def add_pubkey_ref(self, pubkey: str):
        """Adds a reference to a pubkey as a 'p' tag."""
        self.tags.append(["p", pubkey])

    def add_event_ref(self, event_id: str):
        """Adds a reference to an event_id as an 'e' tag."""
        self.tags.append(["e", event_id])

    def verify(self) -> bool:
        pub_key = PublicKey(
            bytes.fromhex("02" + self.public_key), True
        )  # add 02 for schnorr (bip340)
        event_id = Event.compute_id(
            self.public_key, self.created_at, self.kind, self.tags, self.content
        )
        return pub_key.schnorr_verify(
            bytes.fromhex(event_id), bytes.fromhex(self.signature), None, raw=True
        )

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "pubkey": self.public_key,
            "created_at": self.created_at,
            "kind": self.kind,
            "tags": self.tags,
            "content": self.content,
            "sig": self.signature,
        }

    @classmethod
    def from_dict(cls, msg: dict) -> "Event":
        return Event(
            content=msg["content"],
            public_key=msg["pubkey"],
            created_at=msg["created_at"],
            kind=msg["kind"],
            tags=msg["tags"],
            signature=msg["sig"],
        )

    def to_message(self) -> str:
        return json.dumps([ClientMessageType.EVENT, self.to_dict()])


@dataclass
class EncryptedDirectMessage(Event):
    recipient_pubkey: str = None
    cleartext_content: str = None
    reference_event_id: str = None

    def __post_init__(self):
        if self.content:
            self.cleartext_content = self.content
            self.content = None

        if self.recipient_pubkey is None:
            raise Exception("Must specify a recipient_pubkey.")

        self.kind = EventKind.ENCRYPTED_DIRECT_MESSAGE
        super().__post_init__()

        # Must specify the DM recipient's pubkey in a 'p' tag
        self.add_pubkey_ref(self.recipient_pubkey)

        # Optionally specify a reference event (DM) this is a reply to
        if self.reference_event_id:
            self.add_event_ref(self.reference_event_id)

    @property
    def id(self) -> str:
        if self.content is None:
            raise Exception(
                "EncryptedDirectMessage `id` is undefined \
                until its message is encrypted and stored in the `content` field"
            )
        return super().id



import time
from dataclasses import dataclass


@dataclass
class Delegation:
    delegator_pubkey: str
    delegatee_pubkey: str
    event_kind: int
    duration_secs: int = 30 * 24 * 60  # default to 30 days
    signature: str = None  # set in PrivateKey.sign_delegation

    @property
    def expires(self) -> int:
        return int(time.time()) + self.duration_secs

    @property
    def conditions(self) -> str:
        return f"kind={self.event_kind}&created_at<{self.expires}"

    @property
    def delegation_token(self) -> str:
        return f"nostr:delegation:{self.delegatee_pubkey}:{self.conditions}"

    def get_tag(self) -> "list[str]":
        """Called by Event."""
        return [
            "delegation",
            self.delegator_pubkey,
            self.conditions,
            self.signature,
        ]



import json
import logging
import time
from dataclasses import dataclass
from threading import Lock
from typing import Optional

from websocket import (
    WebSocketApp,
    WebSocketConnectionClosedException,
    setdefaulttimeout,
)

from .event import Event
from .filter import Filters
from .message_pool import MessagePool
from .message_type import RelayMessageType
from .subscription import Subscription

logger = logging.getLogger("websocket")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler())

setdefaulttimeout(5)


@dataclass
class RelayPolicy:
    should_read: bool = True
    should_write: bool = True

    def to_json_object(self) -> "dict[str, bool]":
        return {"read": self.should_read, "write": self.should_write}


@dataclass
class RelayProxyConnectionConfig:
    host: Optional[str] = None
    port: Optional[int] = None
    type: Optional[str] = None


class Relay:
    def __init__(
        self,
        url: str,
        policy: RelayPolicy,
        message_pool: MessagePool,
        subscription: "dict[str, Subscription]" = None,
    ) -> None:
        self.url = url
        self.policy = policy
        self.message_pool = message_pool
        self.subscriptions = subscription or {}

        # self.proxy_config: RelayProxyConnectionConfig = RelayProxyConnectionConfig()

        self.lock: Lock = Lock()
        self.ws = WebSocketApp(
            url,
            on_open=self._on_open,
            on_message=self._on_message,
            # on_data=self._on_data,
            on_error=self._on_error,
            on_close=self._on_close,
            on_ping=self._on_ping,
            on_pong=self._on_pong,
        )
        self.active = False

    def connect(self, ssl_options: Optional[dict] = None):
        self.ws.run_forever(
            sslopt=ssl_options,
            # http_proxy_host=self.proxy_config.host,
            # http_proxy_port=self.proxy_config.port,
            # proxy_type=self.proxy_config.type
            ping_interval=60,
            ping_timeout=10,
            ping_payload="2",
            reconnect=30,
        )

    def close(self):
        self.ws.close()

    def publish(self, message: str):
        try:
            self.ws.send(message)
        except WebSocketConnectionClosedException:
            self.active = False
            logger.exception(f"failed to send message to {self.url}")

    def add_subscription(self, id, filters: Filters):
        with self.lock:
            self.subscriptions[id] = Subscription(id, filters)

    def close_subscription(self, id: str) -> None:
        with self.lock:
            self.subscriptions.pop(id)

    def update_subscription(self, id: str, filters: Filters) -> None:
        with self.lock:
            subscription = self.subscriptions[id]
            subscription.filters = filters

    def to_json_object(self) -> dict:
        return {
            "url": self.url,
            "policy": self.policy.to_json_object(),
            "subscriptions": [
                subscription.to_json_object()
                for subscription in self.subscriptions.values()
            ],
        }

    def _on_open(self, class_obj):
        self.active = time.time()
        print(f"OPEN: {self.url}")

    def _on_close(self, class_obj, status_code, message):
        print(f"CLOSE: {self.url} - {message}")
        self.active = False

    def _on_message(self, class_obj, message: str = None):
        self.active = time.time()
        if self._is_valid_message(message):
            self.message_pool.add_message(message, self.url)

    def _on_data(self, class_obj, message: str, data_type, continue_flag):
        print(f"DATA: {self.url} - {message}")

    def _on_error(self, class_obj, error):
        print(f"ERROR: {self.url}")

    def _on_ping(self, class_obj, message):
        pass

    def _on_pong(self, class_obj, message):
        self.active = time.time()

    def _is_valid_message(self, message: str) -> bool:
        message = message.strip("\n")
        if not message or message[0] != "[" or message[-1] != "]":
            return False

        message_json = json.loads(message)
        message_type = message_json[0]
        if not RelayMessageType.is_valid(message_type):
            return False

        if message_type == RelayMessageType.EVENT:
            if not len(message_json) == 3:
                return False

            subscription_id = message_json[1]
            with self.lock:
                if subscription_id not in self.subscriptions:
                    return False

            event = Event.from_dict(message_json[2])

            if not event.verify():
                return False

            with self.lock:
                subscription = self.subscriptions[subscription_id]

            if not subscription.filters.match(event):
                return False

        return True
